# Miscellaneous functions for R
# A. Shipunov, plantago@herba.msu.ru
# v.0.1, 13.04.03
#
# Multiply recoding
#
recode <- function(var, from, to){
x <- as.vector(var)
x.tmp <- x
for (i in 1:length(from)){x <- replace(x, x.tmp == from[i], to[i])}
if(is.factor(var)) factor(x) else x}

#
# Number of variables
#
var.nums <- function(x) paste(seq(1, along=dimnames(x)[[2]]), dimnames(x)[[2]])

#
# Upper valuues of rectangular matrix
#
top.mat <- function(X, level=0.45, N=10, values=TRUE) {
X.nam <- row.names(X)
X.tri <- as.vector(lower.tri(X))
X.rep.g <- rep(X.nam, length(X.nam))[X.tri]
X.rep.e <- rep(X.nam, each=length(X.nam))[X.tri]
X.vec <- as.vector(X)[X.tri]
X.df <- data.frame(Var1=X.rep.g, Var2=X.rep.e, Value=X.vec)
{if (values)
    {X.df <- X.df[abs(X.df$Value) >= level,]
    X.df <- X.df[order(-abs(X.df$Value)),]}
else
    {X.df <- X.df[order(-abs(X.df$Value)),]
    X.df <- X.df[1:N,]}}
row.names(X.df) <- seq(1, along=X.df$Value)
return(X.df)
}

#
# The function has "mode" and "type" arguments (type is typically, but not
# always an object's class attribute). If the user only wants to list
# data.frames, then the command ls.objects(type="data.frame") will list only
# information for data.frames (dputler@scu.edu).
#
ls.objects <- function (pos = 1, pattern, mode = "any", type = "any"){
Object.Name <- ls(pos = pos, envir = as.environment(pos), pattern = pattern)
Object.Mode <- rep("",length(Object.Name))
Object.Type <- rep("",length(Object.Name))
Variables <- rep("-",length(Object.Name))
Observations <- rep("-",length(Object.Name))
for (i in 1:length(Object.Name)){
Object.Mode[[i]] <- mode(get(Object.Name[[i]]))
    if(is.list(get(Object.Name[[i]]))){
	if(is.null(class(get(Object.Name[[i]]))))
	    Object.Type[[i]] <- c("unknown")
	    else {  Object.Attrib <- attributes(get(Object.Name[[i]]))
                    Object.Type[[i]] <- Object.Attrib$class
                    if(Object.Type[[i]]=="data.frame"){
                            Variables[[i]] <- as.character(length(Object.Attrib$names))
                            Observations[[i]] <- as.character(length(Object.Attrib$row.names))
                }}}
    if(is.matrix(get(Object.Name[[i]]))){
	    Object.Attrib <- dim(get(Object.Name[[i]]))
            Object.Type[[i]] <- c("matrix")
            Variables[[i]] <- as.character(Object.Attrib[2])
            Observations[[i]] <- as.character(Object.Attrib[1])
    }
    if(is.vector(get(Object.Name[[i]])) && (Object.Mode[[i]]=="character" || Object.Mode[[i]]=="numeric")){
            Object.Type[[i]] <- c("vector")
            Variables[[i]] <- c("1")
            Observations[[i]] <- as.character(length(get(Object.Name[[i]])))
    }
    if(is.factor(get(Object.Name[[i]]))){
            Object.Type[[i]] <- c("factor")
            Variables[[i]] <- c("1")
            Observations[[i]] <- as.character(length(get(Object.Name[[i]])))
    }
    if(is.function(get(Object.Name[[i]]))) Object.Type[[i]] <- c("function")
}
objList <- data.frame(Object.Name,Object.Mode,Object.Type,Observations,Variables)
if(mode != "any") objList <- objList[objList[["Object.Mode"]] == mode,]
if(type != "any") objList <- objList[objList[["Object.Type"]] == type,]
return(objList)
}
