# Functions from:
# N.S.Rostova. The variability of correlations systems between 
# the morphological characters. 1. Natural populations of
# Leucanthemum vulgare (Asteraceae) // Bot. Journ. 1999. Vol. 84. N 11. P. 50--66
#
# ``Flattened'' correlation matrix
#
cor.vec <- function(X, ...) {X.cor <- cor(X, ...)
	X.nam <- row.names(X.cor)
	X.tri <- as.vector(lower.tri(X.cor))
	X.rep.g <- rep(X.nam, length(X.nam))
	X.rep.e <- rep(X.nam, each=length(X.nam))
	X.pas <- paste(X.rep.g, X.rep.e, sep=" & ")
	X.vec <- as.vector(X.cor)[X.tri]
	names(X.vec) <- X.pas[X.tri]
	return(X.vec)}
#
# Secondary table, consist in correlation coefficients, suitable e.g. for PCA
#
rostova.tbl <- function(X, GROUP, ...) {
old.options <- options(warn=-1)
r.table <- NULL
r.names <- unique(X[[GROUP]])
for (i in r.names) 
	r.table <- cbind (r.table, cor.vec(subset(X, X[[GROUP]]==i)[,-GROUP], ...))
	dimnames(r.table)[[2]] <- r.names
	r.table[is.na(r.table)] <- 0
	r.table <- t(r.table)
	return(r.table)
options(old.options)}
#
# Determination coefficient for correlation matrices
#
coeff.det <- function (X, ...) {X.cor <- cor(X, ...)
X.det <- NULL
X.dim <- dimnames(X.cor)[[2]]
	for (i in 1:length(X.dim)) X.det <- c(X.det, mean(X.cor[,i]^2))
	names(X.det) <- X.dim
	return(X.det)}